// include all sprite images
#include "KK_Bossa_Sprites.h"

// Subfunction: Animation
// LCD animation that pairs with KK Bossa Song
// We play the animtion by minibeats
// recall: there are 16 minibeats in one measure
// note: mb is an integer from [0,num_mb-1]
void Animation(unsigned int mb, unsigned int num_mb) {
  // giant if else to handle all animations minibeat by minibeat
  if (mb == 0) { // start
    // displays: Request which song?

    lcd.clear(); // clear initial screen "Press to Start"
    lcd.home(); // set cursor to (0,0)
    lcd.print(F("Request which"));
    lcd.setCursor(0, 1);
    lcd.print(F("song?"));
    lcd.setCursor(6, 1);
  } else if ( mb < (8 * 16) ) { // up to 9th measure
    // writes out "K.K. Bossa"

    if ((mb % 6) < 3 ) { // if mb mod 6 is 0,1,2
      lcd.noCursor();
    } else { // if mb mod 6 is 3,4,5
      lcd.cursor();
    }
    if ((mb >= 32) && (mb % 8 == 0)) { // after the 2nd measure and if mb is divisible by 8
      char str[] = "K.K. Bossa";
      lcd.print(str[(mb - 32) / 8]);
    }
  } else if ( mb < (16 * 16) ) { // up to 17th measure
    // kk slider will move across the screen from left to right
    mb = mb - (8 * 16); // zero mb, makes things cleaner

    // kk slider (2 frame sprite)
    const uint16_t *kk_slider[2] = {kk_slider0, kk_slider1};
    byte ind;
    if (mb % 12 < 6) { // if mb mod 12 is 0,1,2,3,4,5
      ind = 0;
    } else { // if mb mod 12 is 6,7,8,9,10,11
      ind = 1;
    }

    // right most column of the image
    int col[6] = { -4, -3, -2, -1, 0, 1};

    CreateCharPane16(kk_slider[ind], 15, 0, col[mb % 6] + 12, 31, 1);
    CreateCharPane16(kk_slider[ind], 15, 0, col[mb % 6] + 06, 31, 2);
    CreateCharPane16(kk_slider[ind], 15, 0, col[mb % 6] + 00, 31, 3);
    CreateCharPane16(kk_slider[ind], 15, 8, col[mb % 6] + 12, 31, 4);
    CreateCharPane16(kk_slider[ind], 15, 8, col[mb % 6] + 06, 31, 5);
    CreateCharPane16(kk_slider[ind], 15, 8, col[mb % 6] + 00, 31, 6);

    if ( mb < 6) {// only need to display panes 1 and 4
      lcd.setCursor(0, 0); lcd.write(3);
      lcd.setCursor(0, 1); lcd.write(6);
    } else if ( mb < 12 ) {// only need to display panes 1,2 and 4,5
      lcd.setCursor(0, 0); lcd.write(2); lcd.write(3);
      lcd.setCursor(0, 1); lcd.write(5); lcd.write(6);
    } else if (mb < 16 * 16 ) {
      // display the trail of music notes behind kk slider as he moves along
      if (mb % 6 < 3) { // if mb mod 6 = 0,1,2
        CreateCharPane8(music_note, 7, 0, 00, 31, 7);
      } else { // if mb mod 6 = 3,4,5
        CreateCharPane8(music_note, 7, 0, -1, 31, 7);
      }

      byte cursor_col = mb / 6 - 2; // shift over one each six minibeats

      lcd.setCursor(0, 0); // upper row
      for (int i = 0; i < cursor_col; i++) {
        if (i % 2 == 0 || (mb == 59) || (mb == 60) || (mb == 62) || (mb == 63) || (64 < mb && mb < 73) ) {
          lcd.write(7);
        } else {
          lcd.print(F(" "));
        }
      }
      lcd.write(1); lcd.write(2); lcd.write(3);

      lcd.setCursor(0, 1); // lower row
      for (int i = 0; i < cursor_col; i++) {
        if (i % 2 != 0 || (mb == 59) || (mb == 60) || (mb == 62) || (mb == 63) || (64 < mb && mb < 73) ) {
          lcd.write(7);
        } else {
          lcd.print(F(" "));
        }
      }
      lcd.write(4); lcd.write(5); lcd.write(6);
    }
  } else if (mb < (18 * 16)) { // up to the 19th measure
    // fades the train in and out
    mb = mb - (16 * 16); // zero mb

    if (mb < 8 ) { // fade in
      analogWrite(V0Pin, (8 - mb) * 30); // 240 to 0 in 8 minibeats
    } else if ( mb < 24 ) {
      analogWrite(V0Pin, 0);
    } else { // mb >= 24, fade out
      analogWrite(V0Pin, (mb - 24) * 30); // 0 to 240 in 8 minibeats
    }

    if (mb == 0) { // only need to display the image once
      // train and tracks (still of 7 panes)
      CreateCharPane32(train, 15, 0, 15, 31, 1);
      CreateCharPane32(train, 15, 0, 10, 31, 2);
      CreateCharPane32(train, 15, 0, 05, 31, 3);
      CreateCharPane32(train, 15, 8, 15, 31, 4);
      CreateCharPane32(train, 15, 8, 10, 31, 5);
      CreateCharPane32(train, 15, 8, 05, 31, 6);
      CreateCharPane32(train, 15, 8, 00, 31, 7);

      lcd.clear();
      lcd.setCursor(5, 0); lcd.write(1); lcd.write(2); lcd.write(2); lcd.write(2); lcd.write(3);
      lcd.setCursor(0, 1);
      const byte pos[16] = {7, 7, 7, 7, 7, 4, 5, 5, 5, 6, 7, 7, 7, 7, 7, 7};
      for (int i = 0; i < 16; i++) {
        lcd.write(pos[i]);
      }
    }
  } else if (mb < (20 * 16) ) { // up to the 21st measure
    // fade the villager and tom nook in
    mb = mb - (18 * 16);

    if (mb < 8) { // fade in
      analogWrite(V0Pin, (8 - mb) * 30); // 240 to 0 in 8 minibeats
    }

    const uint16_t *villager;
    if (GENDER) {
      villager = male_villager;
    } else {
      villager = female_villager;
    }

    if (mb == 0) { // only need to display the images once
      // villager (still of 4 panes)
      CreateCharPane16(villager, 15, 0, 7, 31, 0);
      CreateCharPane16(villager, 15, 0, 1, 31, 1);
      CreateCharPane16(villager, 15, 8, 7, 31, 2);
      CreateCharPane16(villager, 15, 8, 1, 31, 3);

      // tom nook (still of 4 panes)
      CreateCharPane16(tom_nook, 15, 0, 7, 31, 4);
      CreateCharPane16(tom_nook, 15, 0, 1, 31, 5);
      CreateCharPane16(tom_nook, 15, 8, 7, 31, 6);
      CreateCharPane16(tom_nook, 15, 8, 1, 31, 7);

      lcd.clear();
      lcd.setCursor(5, 0); lcd.write(byte(0)); lcd.write(1); lcd.print(F(" ")); lcd.print(F(" ")); lcd.write(4); lcd.write(5);
      lcd.setCursor(5, 1); lcd.write(2); lcd.write(3); lcd.print(F(" ")); lcd.print(F(" ")); lcd.write(6); lcd.write(7);
    }
  } else if ( mb < (23 * 16) + 8) { // up to half of the 24th measure
    // display the tent and then show the cost. Fade everything out afterward
    mb = mb - (20 * 16); // zero mb

    if (mb > 48) { // fade out
      analogWrite(V0Pin, (mb - 48) * 30); // 0 to 240 in 8 minibeats
    }

    if ( mb == 0 ) { // only need to display the image once
      // tent (still of 4 panes)
      CreateCharPane16(tent, 15, 0, 05, 31, 0);
      CreateCharPane16(tent, 15, 8, 10, 31, 1);
      CreateCharPane16(tent, 15, 8, 05, 31, 2);
      CreateCharPane16(tent, 15, 8, 00, 31, 3);

      lcd.clear();
      lcd.setCursor(7, 0); lcd.write(byte(0));
      lcd.setCursor(6, 1); lcd.write(1); lcd.write(2); lcd.write(3);
    }

    if (mb == 28) { // only need to display the image once
      // minus a bag of bells
      CreateCharPane16(bells, 11, -6, 0, 31, 4);
      CreateCharPane16(bells, 11, 03, 6, 31, 5);
      CreateCharPane16(bells, 11, 03, 0, 31, 6);
      lcd.setCursor(12, 0); lcd.write(4);
      lcd.setCursor(10, 1); lcd.print(F("-")); lcd.write(5); lcd.write(6);
    }

  } else if ( mb < (26 * 16)) { // up to the 27th measure
    // planting town tree (7 frame animation), then fade out
    mb = mb - ((23 * 16) + 8); // zero mb

    if (mb < 32 ) {
      analogWrite(V0Pin, 0);
    } else { // mb >= 32, fade out
      analogWrite(V0Pin, (mb - 32) * 30); // 0 to 240 in 8 minibeats
    }

    // array of pointers pointing to the villager frames
    const uint16_t *planting[7];
    if (GENDER) { // male villager
      planting[0] = male_planting0;
      planting[1] = male_planting1;
      planting[2] = male_planting2;
      planting[3] = male_planting3;
      planting[4] = male_planting4;
      planting[5] = male_planting5;
      planting[6] = male_planting6;
    } else { // female villager
      planting[0] = female_planting0;
      planting[1] = female_planting1;
      planting[2] = female_planting2;
      planting[3] = female_planting3;
      planting[4] = female_planting4;
      planting[5] = female_planting5;
      planting[6] = female_planting6;
    }

    if ( mb <= 12) {
      CreateCharPane16(planting[ mb / 2 ], 15, 0, 10, 31, 1);
      CreateCharPane16(planting[ mb / 2 ], 15, 0, 05, 31, 2);
      CreateCharPane16(planting[ mb / 2 ], 15, 0, 00, 31, 3);
      CreateCharPane16(planting[ mb / 2 ], 15, 8, 10, 31, 4);
      CreateCharPane16(planting[ mb / 2 ], 15, 8, 05, 31, 5);
      CreateCharPane16(planting[ mb / 2 ], 15, 8, 00, 31, 6);
    }
    if (mb % 2 == 0) {
      lcd.clear();
      lcd.setCursor(6, 0); lcd.write(1); lcd.write(2); lcd.write(3);
      lcd.setCursor(6, 1); lcd.write(4); lcd.write(5); lcd.write(6);
    }
  } else if (mb < (27 * 16) + 8 ) { // up to half of the 28th measure
    // show the town hall, then fade it out
    mb = mb - (26 * 16);

    if (mb < 16 ) {
      analogWrite(V0Pin, 0);
    } else { // mb >= 16, fade out
      analogWrite(V0Pin, (mb - 16) * 30); // 0 to 240 in 8 minibeats
    }

    if (mb == 0) { // town hall, only need to display once
      CreateCharPane32(town_hall, 15, 0, 15, 31, 0);
      CreateCharPane32(town_hall, 15, 0, 10, 31, 1);
      CreateCharPane32(town_hall, 15, 0, 05, 31, 2);
      CreateCharPane32(town_hall, 15, 0, 00, 31, 3);
      CreateCharPane32(town_hall, 15, 8, 15, 31, 4);
      CreateCharPane32(town_hall, 15, 8, 10, 31, 5);
      CreateCharPane32(town_hall, 15, 8, 05, 31, 6);
      CreateCharPane32(town_hall, 15, 8, 00, 31, 7);

      lcd.clear();
      lcd.setCursor(6, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(3);
      lcd.setCursor(6, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
    }

  } else if (mb < (31 * 16) + 8 ) { // up to half of the 32nd measure
    // show main street.
    mb = mb - ((27 * 16) + 8); // zero mb
    mb = mb / 3; // display every 3 minibeats

    if (mb < 20) {
      analogWrite(V0Pin, 0);
    } else { // mb >= 20
      analogWrite(V0Pin, 255); // turn contrast off for the next section
    }

    const uint32_t *main_street[4] = {main_street0, main_street1, main_street2, main_street3};

    if (mb < 3) {
      CreateCharPane32(main_street[0], 15, 0, 15, 31, 0);
      CreateCharPane32(main_street[0], 15, 0, 10, 31, 1);
      CreateCharPane32(main_street[0], 15, 0, 05, 31, 2);
      CreateCharPane32(main_street[0], 15, 8, 15, 31, 4);
      CreateCharPane32(main_street[0], 15, 8, 10, 31, 5);
      CreateCharPane32(main_street[0], 15, 8, 05, 31, 6);

      lcd.clear();
      if (mb == 0) {
        lcd.setCursor(0, 0); lcd.write(byte(0));
        lcd.setCursor(0, 1); lcd.write(4);
      } else if (mb == 1) {
        lcd.setCursor(0, 0); lcd.write(byte(0)); lcd.write(1);
        lcd.setCursor(0, 1); lcd.write(4); lcd.write(5);
      } else { // mb == 2
        lcd.setCursor(0, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2);
        lcd.setCursor(0, 1); lcd.write(4); lcd.write(5); lcd.write(6);
      }
    } else if (mb < 16) {
      mb = mb - 3;

      byte frame = mb / 4; // this frame and the next are to be displayed
      byte col[7] = {15, 10, 5, 0, 15, 10, 5}; // right most column of image pane
      byte ind = mb % 4; // index for col

      for (int i = 0; i < 4; i++) {
        if (i < (4 - ind)) { // pane creation from the current frame
          CreateCharPane32(main_street[frame], 15, 0, col[ind+i], 31, i);
          CreateCharPane32(main_street[frame], 15, 8, col[ind+i], 31, i+4);
        } else { // pane creation from the next frame
          CreateCharPane32(main_street[frame + 1], 15, 0, col[ind+i], 31, i);
          CreateCharPane32(main_street[frame + 1], 15, 8, col[ind+i], 31, i+4);
        }
      }

      lcd.clear();
      lcd.setCursor(mb, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(3);
      lcd.setCursor(mb, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
    } else if (mb < 20) {
      CreateCharPane32(main_street[3], 15, 0, 10, 31, 1);
      CreateCharPane32(main_street[3], 15, 0, 05, 31, 2);
      CreateCharPane32(main_street[3], 15, 0, 00, 31, 3);
      CreateCharPane32(main_street[3], 15, 8, 10, 31, 5);
      CreateCharPane32(main_street[3], 15, 8, 05, 31, 6);
      CreateCharPane32(main_street[3], 15, 8, 00, 31, 7);

      lcd.clear();
      if (mb == 16) {
        lcd.setCursor(13, 0); lcd.write(1); lcd.write(2); lcd.write(3);
        lcd.setCursor(13, 1); lcd.write(5); lcd.write(6); lcd.write(7);
      } else if (mb == 17) {
        lcd.setCursor(14, 0); lcd.write(2); lcd.write(3);
        lcd.setCursor(14, 1);  lcd.write(6); lcd.write(7);
      } else if (mb == 18) {
        lcd.setCursor(15, 0); lcd.write(3);
        lcd.setCursor(15, 1); lcd.write(7);
      } else { // mb == 19
        // do nothing
      }
    }
  } else if (mb < (33 * 16) + 8) { // up to half of the 34th measure
    // fade in the villager giving bells to tom nook.
    mb = mb - ((31 * 16) + 8);

    if (mb < 8) { // fade in
      analogWrite(V0Pin, (8 - mb) * 30); // 240 to 0 in 8 minibeats
    }

    const uint16_t *villager;
    if (GENDER) { // male villager
      villager = male_villager;
    } else { // female villager
      villager = female_villager;
    }

    if (mb == 0) { // only display the bag of bells once
      CreateCharPane16(bells, 11, -6, 0, 31, 5);
      CreateCharPane16(bells, 11, 03, 6, 31, 6);
      CreateCharPane16(bells, 11, 03, 0, 31, 7);
      lcd.clear();
      lcd.setCursor(8, 0); lcd.write(5); lcd.setCursor(7, 1); lcd.write(6); lcd.write(7);
    }

    if (mb < 8) {
      CreateCharPane16(villager, 16, 0, 7, 31, 1);
      CreateCharPane16(villager, 16, 0, 1, 31, 2);
      CreateCharPane16(villager, 16, 8, 7, 31, 3);
      CreateCharPane16(villager, 16, 8, 1, 31, 4);
      lcd.setCursor(5, 0); lcd.write(1); lcd.write(2);
      lcd.setCursor(5, 1); lcd.write(3); lcd.write(4);
    } else if (mb < 13 ) {
      byte select1[5] = {31, 15, 7, 3, 1};
      int i = mb - 8; // 0 -> 4
      CreateCharPane16(villager, 16, 0, 7, select1[i], 1);
      CreateCharPane16(villager, 16, 8, 7, select1[i], 3);
      lcd.setCursor(5, 0); lcd.write(1);
      lcd.setCursor(5, 1); lcd.write(3);
    } else if (mb == 13) {
      lcd.setCursor(5, 0); lcd.print(F(" "));
      lcd.setCursor(5, 1); lcd.print(F(" "));
    } else if (mb < 19) {
      byte select1[5] = {31, 15, 7, 3, 1};
      byte select2[5] = {16, 24, 28, 30, 31};
      int i = mb - 14; // 0 -> 4
      CreateCharPane16(villager, 16, 0, 1, select1[i], 2);
      CreateCharPane16(villager, 16, 8, 1, select1[i], 4);
      CreateCharPane16(tom_nook, 16, 0, 7, select2[i], 1);
      CreateCharPane16(tom_nook, 16, 8, 7, select2[i], 3);
      lcd.setCursor(6, 0); lcd.write(2); lcd.setCursor(9, 0); lcd.write(1);
      lcd.setCursor(6, 1); lcd.write(4); lcd.setCursor(9, 1); lcd.write(3);
    } else if (mb == 19) {
      lcd.setCursor(6, 0); lcd.print(F(" "));
      lcd.setCursor(6, 1); lcd.print(F(" "));
    } else if (mb < 24) {
      byte select2[5] = {16, 24, 28, 30, 31};
      int i = mb - 20; // 0 -> 4
      CreateCharPane16(tom_nook, 16, 0, 1, select2[i], 2);
      CreateCharPane16(tom_nook, 16, 8, 1, select2[i], 4);
      lcd.setCursor(10, 0); lcd.write(2);
      lcd.setCursor(10, 1); lcd.write(4);
    }
  } else if (mb < (35 * 16)) { // up the 36th measure
    // display the house, fade out
    mb = mb - ((33 * 16) + 8);

    if (mb > 16) { // fade out
      analogWrite(V0Pin, (mb - 16) * 30); // 0 to 240 in 8 minibeats
    }

    if (mb == 0) { // only need to display the small house once
      CreateCharPane16(small_house, 15, 0, 10, 31, 1);
      CreateCharPane16(small_house, 15, 0, 05, 31, 2);
      CreateCharPane16(small_house, 15, 0, 00, 31, 3);
      CreateCharPane16(small_house, 15, 8, 10, 31, 4);
      CreateCharPane16(small_house, 15, 8, 05, 31, 5);
      CreateCharPane16(small_house, 15, 8, 00, 31, 6);
      lcd.clear();
      lcd.setCursor(6, 0); lcd.write(1); lcd.write(2); lcd.write(3);
      lcd.setCursor(6, 1); lcd.write(4); lcd.write(5); lcd.write(6);
    }
  } else if (mb < (39 * 16) ) { // up to the 40th measure
    // Have the villager daydream of a dream house, then display bags of bells
    mb = mb - (35 * 16);

    analogWrite(V0Pin, 0);

    if (mb == 0 ) { // villager's head
      if (GENDER) { // male
        CreateCharPane16(male_villager, 15, 0, 7, 31, 1);
        CreateCharPane16(male_villager, 15, 0, 1, 31, 2);
      } else { // female
        CreateCharPane16(female_villager, 15, 0, 7, 31, 1);
        CreateCharPane16(female_villager, 15, 0, 1, 31, 2);
      }
      lcd.clear();
      lcd.setCursor(5, 1); lcd.write(1); lcd.write(2);
    } else if (mb == 10) { // thinking clouds
      CreateCharPane8(cloud0, 2, -1, -1, 31, 3);
      lcd.setCursor(7, 1); lcd.write(3);
    } else if (mb == 12) {
      CreateCharPane8(cloud0, 2, -4, -1, 31, 4);
      lcd.setCursor(8, 0); lcd.write(4);
    } else if (mb == 14) {
      CreateCharPane8(cloud1, 3, -1, 0, 31, 5);
      lcd.setCursor(9, 0); lcd.write(5);
    } else if (mb == 16) { // dream house
      CreateCharPane32(dream_house, 15, 0, 15, 31, 0);
      CreateCharPane32(dream_house, 15, 0, 10, 31, 1);
      CreateCharPane32(dream_house, 15, 0, 05, 31, 2);
      CreateCharPane32(dream_house, 15, 0, 00, 31, 3);
      CreateCharPane32(dream_house, 15, 8, 15, 31, 4);
      CreateCharPane32(dream_house, 15, 8, 10, 31, 5);
      CreateCharPane32(dream_house, 15, 8, 05, 31, 6);
      CreateCharPane32(dream_house, 15, 8, 00, 31, 7);
      lcd.clear();
      lcd.setCursor(6, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(3);
      lcd.setCursor(6, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
    }

    // bag of bells transition
    if (mb >= 46) {
      CreateCharPane16(bells, 11, -6, 0, 31, 1);
      CreateCharPane16(bells, 11, 03, 6, 31, 2);
      CreateCharPane16(bells, 11, 03, 0, 31, 3);

      if (mb == 46) {
        lcd.clear(); lcd.setCursor(0, 0);
        for (int i = 0; i < 16; i++) {
          lcd.write(2); lcd.write(3);
        }
        lcd.setCursor(0, 1);
        for (int i = 0; i < 16; i++) {
          lcd.write(3); lcd.write(2);
        }
      } else if (mb <= 54) {
        lcd.setCursor(0, 0);
        for (int i = 0; i < (mb - 46); i++) {
          lcd.write(1); lcd.print(F(" "));
        }
      } else {
        lcd.setCursor(16, 0); lcd.rightToLeft();
        for (int i = 0; i < (mb - 54); i++) {
          lcd.print(F(" ")); lcd.print(F(" "));
        }
        lcd.setCursor(16, 1);
        for (int i = 0; i < (mb - 54); i++) {
          lcd.print(F(" ")); lcd.print(F(" "));
        }
        lcd.leftToRight();
      }
    }
  } else if (mb < (41 * 16)) { // up to the 42st measure
    // hitting rocks
    mb = mb - (39 * 16);

    const uint16_t *villager_hitting_rock[3];
    if (GENDER) { // male
      villager_hitting_rock[0] = male_hitting_rock0;
      villager_hitting_rock[1] = male_hitting_rock1;
      villager_hitting_rock[2] = male_hitting_rock2;
    } else { // female
      villager_hitting_rock[0] = female_hitting_rock0;
      villager_hitting_rock[1] = female_hitting_rock1;
      villager_hitting_rock[2] = female_hitting_rock2;
    }

    if (mb < 16) { // hitting a rock
      CreateCharPane16(villager_hitting_rock[0], 15, 0, 11, 31, 1);
      CreateCharPane16(villager_hitting_rock[0], 15, 0, 05, 31, 2);
      CreateCharPane16(villager_hitting_rock[0], 15, 8, 11, 31, 3);
      CreateCharPane16(villager_hitting_rock[0], 15, 8, 05, 31, 4);
      CreateCharPane16(villager_hitting_rock[0], 15, 8, -1, 31, 5);
      CreateCharPane8(rock, 7, 0, 5, 31, 6);
      CreateCharPane8(rock, 7, 0, 0, 31, 7);

      if (mb == 0) {
        lcd.clear();
      } else if (mb == 7) {
        lcd.clear();
        lcd.setCursor(0, 0); lcd.write(1); lcd.write(2);
        lcd.setCursor(0, 1); lcd.write(3); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
      } else if (mb == 13) {
        lcd.clear();
        lcd.setCursor(4, 0); lcd.write(1); lcd.write(2);
        lcd.setCursor(4, 1); lcd.write(3); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
      } else if (mb == 14) {
        lcd.clear();
        lcd.setCursor(8, 0); lcd.write(1); lcd.write(2);
        lcd.setCursor(8, 1); lcd.write(3); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
      }
    } else if (mb >= 16) { // money rock
      mb = mb - 16;

      // set up sequences for each minibeat
      byte frame[16] = {0, 0, 0, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2}; // frame sequence
      byte col[16] = { -1, 1, 3, 3, 1, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}; // right most column of the sprite image

      // note: first three indices will not be used
      const uint16_t *money_rock[16] = {0, 0, money_rock0, money_rock1, money_rock2, money_rock3, money_rock4, money_rock5, money_rock6, money_rock7, money_rock8, money_rock9, money_rock10, money_rock10, money_rock10, money_rock10};

      // pane creation. note how the panes are changed
      CreateCharPane16(villager_hitting_rock[frame[mb]], 15, 0, col[mb] + 12, 31, 0);
      CreateCharPane16(villager_hitting_rock[frame[mb]], 15, 0, col[mb] + 06, 31, 1);
      CreateCharPane16(villager_hitting_rock[frame[mb]], 15, 8, col[mb] + 12, 31, 3);
      CreateCharPane16(villager_hitting_rock[frame[mb]], 15, 8, col[mb] + 06, 31, 4);
      if  (mb < 8) {
        CreateCharPane16(villager_hitting_rock[frame[mb]], 15, 0, col[mb] + 00, 31, 2);
        CreateCharPane16(villager_hitting_rock[frame[mb]], 15, 8, col[mb] + 00, 31, 5);
        if (mb < 2) {
          CreateCharPane8(rock, 7, 0, 5, 31, 7);
          lcd.clear();
          lcd.setCursor(12, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2);
          lcd.setCursor(12, 1); lcd.write(3); lcd.write(4); lcd.write(5); lcd.write(7);
        } else  {
          CreateCharPane16(money_rock[mb], 15, 0, 0, 31, 6);
          CreateCharPane16(money_rock[mb], 15, 8, 0, 31, 7);
          lcd.clear();
          lcd.setCursor(12, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(6);
          lcd.setCursor(12, 1); lcd.write(3); lcd.write(4); lcd.write(5); lcd.write(7);
        }
      } else { // mb >= 8
        CreateCharPane16(money_rock[mb], 15, 0, 5, 31, 2);
        CreateCharPane16(money_rock[mb], 15, 0, 0, 31, 6);
        CreateCharPane16(money_rock[mb], 15, 8, 5, 31, 5);
        CreateCharPane16(money_rock[mb], 15, 8, 0, 31, 7);

        lcd.setCursor(12, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(6);
        lcd.setCursor(12, 1); lcd.write(3); lcd.write(4); lcd.write(5); lcd.write(7);
      }
    }
  } else if (mb < (43 * 16)) { // up to the 44st measure
    // obtaining fossils
    mb = mb - (41 * 16);

    const uint32_t *finding_fossil;
    if (GENDER) {
      finding_fossil = male_finding_fossil;
    } else {
      finding_fossil = female_finding_fossil;
    }

    if (mb < 16) { // finding a fossil
      CreateCharPane32(finding_fossil, 15, 0, 15, 31, 0);
      CreateCharPane32(finding_fossil, 15, 0, 10, 31, 1);
      CreateCharPane32(finding_fossil, 15, 0, 05, 31, 2);
      CreateCharPane32(finding_fossil, 15, 0, 00, 31, 3);
      CreateCharPane32(finding_fossil, 15, 8, 15, 31, 4);
      CreateCharPane32(finding_fossil, 15, 8, 10, 31, 5);
      CreateCharPane32(finding_fossil, 15, 8, 05, 31, 6);
      CreateCharPane32(finding_fossil, 15, 8, 00, 31, 7);

      if (mb == 0) {
        lcd.clear();
      } else if (mb == 7) {
        lcd.clear();
        lcd.setCursor(0, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(3);
        lcd.setCursor(0, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
      } else if (mb == 11) {
        lcd.clear();
        lcd.setCursor(4, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(3);
        lcd.setCursor(4, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
      } else if (mb == 14) {
        lcd.clear();
        lcd.setCursor(8, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(3);
        lcd.setCursor(8, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
      }
    } else { // mb >= 16
      const uint32_t *dino;
      if (mb < 20) {
        dino = dino0;
      } else if (mb < 25) {
        dino = dino1;
      } else { // mb < 32
        dino = dino2;
      }
      CreateCharPane32(dino, 15, 0, 15, 31, 0);
      CreateCharPane32(dino, 15, 0, 10, 31, 1);
      CreateCharPane32(dino, 15, 0, 05, 31, 2);
      CreateCharPane32(dino, 15, 0, 00, 31, 3);
      CreateCharPane32(dino, 15, 8, 15, 31, 4);
      CreateCharPane32(dino, 15, 8, 10, 31, 5);
      CreateCharPane32(dino, 15, 8, 05, 31, 6);
      CreateCharPane32(dino, 15, 8, 00, 31, 7);
      lcd.clear();
      lcd.setCursor(12, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(3);
      lcd.setCursor(12, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
    }
  } else if (mb < (47 * 16)) { // up to the 48th measure
    // picking up fruit, then a bee hive appears and bees chase the villager while then fading out
    mb = mb - (43 * 16);

    if (mb > 55) { // fade out
      analogWrite(V0Pin, (mb - 55) * 30); // 0 to 240 in 8 minibeats
    }

    const uint16_t *villager[2];
    if (GENDER) {
      villager[0] = male_villager;
      villager[1] = male_villager_left;
    } else {
      villager[0] = female_villager;
      villager[1] = female_villager_left;
    }

    const uint16_t *bees_coming_out[3] = {bees_coming_out0, bees_coming_out1, bees_coming_out2};
    const uint16_t *bees[3] = {bees0, bees1, bees2};

    if (mb < 17) { // fruit
      CreateCharPane16(villager[0], 15, 0, 7, 31, 0);
      CreateCharPane16(villager[0], 15, 0, 1, 31, 1);
      CreateCharPane16(villager[0], 15, 8, 7, 31, 2);
      CreateCharPane16(villager[0], 15, 8, 1, 31, 3);

      if (mb == 0) {
        lcd.clear();
      } else if (mb == 7) { // banana
        CreateCharPane32(fruit, 7, 0, 25, 31, 4);
        CreateCharPane32(fruit, 7, 0, 20, 31, 5);
        lcd.clear();
        lcd.setCursor(0, 0); lcd.write(byte(0)); lcd.write(1);
        lcd.setCursor(0, 1); lcd.write(2); lcd.write(3); lcd.write(4); lcd.write(5);
      } else if (mb == 11) { // pear
        CreateCharPane32(fruit, 7, 0, 15, 31, 4);
        CreateCharPane32(fruit, 7, 0, 10, 31, 5);
        lcd.clear();
        lcd.setCursor(4, 0); lcd.write(byte(0)); lcd.write(1);
        lcd.setCursor(4, 1); lcd.write(2); lcd.write(3); lcd.write(4); lcd.write(5);
      } else if (mb == 14) { // cherry
        CreateCharPane32(fruit, 7, 0, 5, 31, 4);
        CreateCharPane32(fruit, 7, 0, 0, 31, 5);
        lcd.clear();
        lcd.setCursor(8, 0); lcd.write(byte(0)); lcd.write(1);
        lcd.setCursor(8, 1); lcd.write(2); lcd.write(3); lcd.write(4); lcd.write(5);
      } else if (mb == 16) { // bee hive
        CreateCharPane8(bee_hive, 7, 0, 4, 31, 6);
        CreateCharPane8(bee_hive, 7, 0, 0, 31, 7);
        lcd.clear();
        lcd.setCursor(12, 0); lcd.write(byte(0)); lcd.write(1);
        lcd.setCursor(12, 1); lcd.write(2); lcd.write(3); lcd.write(6); lcd.write(7);
      }
    } else if (mb < 35) { // villager steps back a few steps, then bees coming out of the hive
      if (mb < 26) {
        // do nothing
      } else if (mb < 32) { // villager steping back a few steps
        // right most column of the image
        int col[6] = { 0, -1, -2, -3, -4, -5};

        CreateCharPane16(villager[0], 15, 0, col[mb - 26] + 12, 31, 0);
        CreateCharPane16(villager[0], 15, 0, col[mb - 26] + 06, 31, 1);
        CreateCharPane16(villager[0], 15, 0, col[mb - 26] + 00, 31, 2);
        CreateCharPane16(villager[0], 15, 8, col[mb - 26] + 12, 31, 3);
        CreateCharPane16(villager[0], 15, 8, col[mb - 26] + 06, 31, 4);
        CreateCharPane16(villager[0], 15, 8, col[mb - 26] + 00, 31, 5);
        lcd.setCursor(11, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2);
        lcd.setCursor(11, 1); lcd.write(3); lcd.write(4); lcd.write(5);
      } else if (mb < 35) { // bees come out of the hive
        CreateCharPane16(bees_coming_out[mb - 32], 15, 0, 5, 31, 2);
        CreateCharPane16(bees_coming_out[mb - 32], 15, 0, 0, 31, 5);
        CreateCharPane16(bees_coming_out[mb - 32], 15, 8, 5, 31, 6);
        CreateCharPane16(bees_coming_out[mb - 32], 15, 8, 0, 31, 7);
        lcd.setCursor(13, 0); lcd.print(F(" ")); lcd.write(2); lcd.write(5);
        lcd.setCursor(13, 1); lcd.print(F(" ")); lcd.write(6); lcd.write(7);
      }
    } else if (mb < 64) { // villager gets chased by bees
      if (mb < 37) {
        lcd.clear();
        lcd.setCursor(11, 0); lcd.write(byte(0)); lcd.write(1);
        lcd.setCursor(11, 1); lcd.write(3); lcd.write(4);

        CreateCharPane16(bees[mb % 3], 8, 0, 04, 31, 6);
        CreateCharPane16(bees[mb % 3], 8, 0, -2, 31, 7);
        lcd.setCursor(14, 0); lcd.write(6); lcd.write(7);
      } else if (mb < 64) {
        mb = mb - 37;

        // right most column of villager image
        int col1[2] = { 0, -3 };

        CreateCharPane16(villager[1], 15, 0, col1[mb % 2] + 12, 31, 0);
        CreateCharPane16(villager[1], 15, 0, col1[mb % 2] + 06, 31, 1);
        CreateCharPane16(villager[1], 15, 0, col1[mb % 2] + 00, 31, 2);
        CreateCharPane16(villager[1], 15, 8, col1[mb % 2] + 12, 31, 3);
        CreateCharPane16(villager[1], 15, 8, col1[mb % 2] + 06, 31, 4);
        CreateCharPane16(villager[1], 15, 8, col1[mb % 2] + 00, 31, 5);

        // right most column of bee image
        int col2[2] = { -2, -5};

        CreateCharPane16(bees[mb % 3], 8, 0, col2[mb % 2] + 06, 31, 6);
        CreateCharPane16(bees[mb % 3], 8, 0, col2[mb % 2] + 00, 31, 7);

        lcd.clear();
        if (mb / 2 < 11) {
          lcd.setCursor(10 - mb / 2, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2);
          lcd.setCursor(10 - mb / 2, 1); lcd.write(3); lcd.write(4); lcd.write(5);
        } else if (mb / 2 == 11) {
          lcd.setCursor(0, 0); lcd.write(1); lcd.write(2);
          lcd.setCursor(0, 1); lcd.write(4); lcd.write(5);
        } else if (mb / 2 == 12) {
          lcd.setCursor(0, 0); lcd.write(2);
          lcd.setCursor(0, 1); lcd.write(5);
        } // do nothing otherwise

        lcd.setCursor(14 - mb / 2, 0); lcd.write(6); lcd.write(7);
      }
    }
  } else if (mb < (49 * 16)) { // up to the 50th measure
    // fishing
    mb = mb - (47 * 16);

    analogWrite(V0Pin, 0);

    const uint32_t *fishing[6];
    if (GENDER) {
      fishing[0] = male_fishing0;
      fishing[1] = male_fishing1;
      fishing[2] = male_fishing2;
      fishing[3] = male_fishing3;
      fishing[4] = male_fishing4;
      fishing[5] = male_fishing5;
    } else {
      fishing[0] = female_fishing0;
      fishing[1] = female_fishing1;
      fishing[2] = female_fishing2;
      fishing[3] = female_fishing3;
      fishing[4] = female_fishing4;
      fishing[5] = female_fishing5;
    }

    // sequence of frames
    byte frame[16] = {0, 0, 0, 1, 0, 1, 0, 1, 2, 2, 3, 4, 5, 5, 5, 5};
    byte ind;
    if (mb < 16) {
      ind = 0;
    } else { // mb >=16
      ind = mb - 16;
    }

    CreateCharPane32(fishing[frame[ind]], 15, 0, 15, 31, 0);
    CreateCharPane32(fishing[frame[ind]], 15, 0, 10, 31, 1);
    CreateCharPane32(fishing[frame[ind]], 15, 0, 05, 31, 2);
    CreateCharPane32(fishing[frame[ind]], 15, 0, 00, 31, 3);
    CreateCharPane32(fishing[frame[ind]], 15, 8, 15, 31, 4);
    CreateCharPane32(fishing[frame[ind]], 15, 8, 10, 31, 5);
    CreateCharPane32(fishing[frame[ind]], 15, 8, 05, 31, 6);
    CreateCharPane32(fishing[frame[ind]], 15, 8, 00, 31, 7);

    if (mb == 0) {
      lcd.clear();
    } else if (mb == 7) {
      lcd.clear();
      lcd.setCursor(0, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(3);
      lcd.setCursor(0, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
    } else if (mb == 11) {
      lcd.clear();
      lcd.setCursor(4, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(3);
      lcd.setCursor(4, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
    } else if (mb == 14) {
      lcd.clear();
      lcd.setCursor(8, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(3);
      lcd.setCursor(8, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
    } else if (mb >= 16) {
      lcd.clear();
      lcd.setCursor(12, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2); lcd.write(3);
      lcd.setCursor(12, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
    }
  } else if (mb < (51 * 16)) { // up to the 52th measure
    // shooting down a balloon present
    mb = mb - (49 * 16);

    const uint16_t *villager;
    if (GENDER) {
      villager = male_villager_left;
    } else {
      villager = female_villager_left;
    }

    if (mb < 24) { // balloon present and villager
      const uint16_t *balloon_present[4] = {balloon_present0, balloon_present1, balloon_present2, balloon_present3};

      byte frame[8] = {0, 0, 0, 0, 0, 1, 2, 3};
      byte ind = 0;
      if (mb >= 16) {
        ind = mb - 16;
      }

      CreateCharPane16(balloon_present[frame[ind]], 15, 0, 10, 31, 0);
      CreateCharPane16(balloon_present[frame[ind]], 15, 0, 05, 31, 1);
      CreateCharPane16(balloon_present[frame[ind]], 15, 0, 00, 31, 2);
      CreateCharPane16(balloon_present[frame[ind]], 15, 8, 10, 31, 3);
      CreateCharPane16(balloon_present[frame[ind]], 15, 8, 05, 31, 4);
      CreateCharPane16(balloon_present[frame[ind]], 15, 8, 00, 31, 5);
      CreateCharPane16(villager, 15, 0, 6, 31, 6);
      CreateCharPane16(villager, 15, 0, 0, 31, 7);

      if (mb == 0) {
        lcd.clear();
      } else if (mb == 6) { // notice pane 2 is the same as pane 0
        lcd.clear();
        lcd.setCursor(0, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(byte(2));
        lcd.setCursor(0, 1); lcd.write(3); lcd.write(4); lcd.write(5);
      } else if (mb == 8) {
        lcd.clear();
        lcd.setCursor(3, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(byte(2));
        lcd.setCursor(3, 1); lcd.write(3); lcd.write(4); lcd.write(5);
      } else if (mb == 11) {
        lcd.clear();
        lcd.setCursor(6, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(byte(2));
        lcd.setCursor(6, 1); lcd.write(3); lcd.write(4); lcd.write(5);
      } else if (mb >= 14) {
        lcd.clear();
        lcd.setCursor(9, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(byte(2));
        lcd.setCursor(9, 1); lcd.write(3); lcd.write(4); lcd.write(5);
        if (mb >= 16) {
          lcd.setCursor(13, 1); lcd.write(6); lcd.write(7);
        }
      }
    } else if (mb < 28) { // present
      byte row[5] = { -9, -7, -5, -3, 0};
      byte ind = 0;
      if (mb >= 23) {
        ind = mb - 23;
      }

      CreateCharPane8(present, 15, -3, 0, 31, 1);
      CreateCharPane16(villager, 15, row[ind] - 0, 6, 31, 4);
      CreateCharPane16(villager, 15, row[ind] - 0, 0, 31, 5);
      CreateCharPane16(villager, 15, row[ind] + 8, 6, 31, 6);
      CreateCharPane16(villager, 15, row[ind] + 8, 0, 31, 7);

      lcd.clear();
      lcd.setCursor(10, 1); lcd.write(1);
      lcd.setCursor(13, 0); lcd.write(4); lcd.write(5);
      lcd.setCursor(13, 1); lcd.write(6); lcd.write(7);
    }

  } else if (mb < (58 * 16)) { // up to the 59th measure
    // a butterfly flies across the screen whlie the villager attempts to catch it. In the end, the villager catches it. Then fade eveything out
    mb = mb - (51 * 16);

    if (mb > 104) { // fade out
      analogWrite(V0Pin, (mb - 104) * 30); // 0 to 240 in 8 minibeats
    }

    const uint8_t *butterfly[] = {butterfly0, butterfly1, butterfly2, butterfly3, butterfly4, butterfly5, butterfly6, butterfly7};

    signed char row[8] = { -3, -1, 0, -2, 0, -1, -3, -2 };
    byte frame1 = (mb / 3) % 8;
    if (mb < 92) {
      CreateCharPane8(butterfly[frame1], 4, row[frame1], 0, 31, 0);
    } else {
      CreateCharPane8(butterfly[2], 4, -1, 0, 31, 0); // butterfly is caught
    }

    const uint32_t *villager_and_net[3];
    if (GENDER) {
      villager_and_net[0] = male_villager_using_net0;
      villager_and_net[1] = male_villager_using_net1;
      villager_and_net[2] = male_villager_using_net2;
    } else {
      villager_and_net[0] = female_villager_using_net0;
      villager_and_net[1] = female_villager_using_net1;
      villager_and_net[2] = female_villager_using_net2;
    }

    byte frame2;
    if ((mb >= 44 && mb < 48) || (mb >= 60 && mb < 64) || (mb >= 76 && mb < 80) || (mb == 92)) {
      frame2 = 0;
    } else if ((mb >= 48 && mb < 54) || (mb >= 64 && mb < 70) || (mb >= 80 && mb < 86)) {
      frame2 = 1;
    } else if ((mb >= 54 && mb < 60) || (mb >= 70 && mb < 76) || (mb >= 86 && mb < 92)) {
      frame2 = 2;
    }

    if (frame2 == 0) {
      CreateCharPane32(villager_and_net[frame2], 15, 0, 15, 31, 1);
      CreateCharPane32(villager_and_net[frame2], 15, 0, 10, 31, 2);
      CreateCharPane32(villager_and_net[frame2], 15, 0, 05, 31, 3);
      CreateCharPane32(villager_and_net[frame2], 15, 0, 00, 31, 4);
      CreateCharPane32(villager_and_net[frame2], 15, 8, 10, 31, 5);
      CreateCharPane32(villager_and_net[frame2], 15, 8, 05, 31, 6);
    } else { // else frame==1,2
      CreateCharPane32(villager_and_net[frame2], 15, 0, 10, 31, 1);
      CreateCharPane32(villager_and_net[frame2], 15, 0, 05, 31, 2);
      CreateCharPane32(villager_and_net[frame2], 15, 0, 00, 31, 3);
      CreateCharPane32(villager_and_net[frame2], 15, 8, 15, 31, 4);
      CreateCharPane32(villager_and_net[frame2], 15, 8, 10, 31, 5);
      CreateCharPane32(villager_and_net[frame2], 15, 8, 05, 31, 6);
      CreateCharPane32(villager_and_net[frame2], 15, 8, 00, 31, 7);
    }

    // record the cursor col for villager placement
    byte cursor_col;

    // butterfly
    if (mb < 12) {
      lcd.clear();
    } else if (mb < 16) { // mb starts at 12
      lcd.clear();
      lcd.setCursor(1, 0); lcd.write(byte(0));
    } else if (mb < 20) { // mb starts at 16
      lcd.clear();
      lcd.setCursor(5, 0); lcd.write(byte(0));
    } else if (mb < 54) { // mb starts at 20
      lcd.clear();
      cursor_col = 9;
      lcd.setCursor(9, 0); lcd.write(byte(0));
    } else if (mb < 60) { // mb starts at 54
      lcd.clear();
      cursor_col = 8;
      lcd.setCursor(8, 0); lcd.write(byte(0));
    } else if (mb < 70) { // mb starts at 60
      lcd.clear();
      cursor_col = 5;
      lcd.setCursor(5, 0); lcd.write(byte(0));
    } else if (mb < 76) { // mb starts at 70
      lcd.clear();
      cursor_col = 4;
      lcd.setCursor(4, 0); lcd.write(byte(0));
    } else if (mb < 86) { // mb starts at 76
      lcd.clear();
      cursor_col = 1;
      lcd.setCursor(1, 0); lcd.write(byte(0));
    } else if (mb < 92) { // mb starts at 86
      lcd.clear();
    } else if (mb == 92) { // mb at 92
      // butterfly caught
      lcd.clear();
      cursor_col = 1;
      lcd.setCursor(2, 1); lcd.write(byte(0));
    }

    // villager
    if ((mb >= 44 && mb < 48) || (mb >= 60 && mb < 64) || (mb >= 76 && mb < 80) || (mb == 92) ) {
      lcd.setCursor(cursor_col + 1, 0); lcd.write(1); lcd.write(2); lcd.write(3); lcd.write(4);
      lcd.setCursor(cursor_col + 2, 1); lcd.write(5); lcd.write(6);
    } else if ((mb >= 48 && mb < 54) || (mb >= 64 && mb < 70) || (mb >= 80 && mb < 86) || (mb >= 54 && mb < 60) || (mb >= 70 && mb < 76) || (mb >= 86 && mb < 92)) {
      if (frame2 == 1) {
        lcd.setCursor(cursor_col + 1, 0); lcd.write(1); lcd.write(2); lcd.write(3);
        lcd.setCursor(cursor_col, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
      } else if (frame2 == 2) {
        lcd.setCursor(cursor_col + 2, 0); lcd.write(1); lcd.write(2); lcd.write(3);
        lcd.setCursor(cursor_col + 1, 1); lcd.write(4); lcd.write(5); lcd.write(6); lcd.write(7);
      }
    }
  } else if (mb < (66 * 16) ) { // up to the 67th measure
    // town tree grows over time. fade in on start and fade out on end.
    mb = mb - (58 * 16);

    if (mb < 8) { // fade in
      analogWrite(V0Pin, (8 - mb) * 30); // 240 to 0 in 8 minibeats
    } else if (mb > 120) { // fade out
      analogWrite(V0Pin, (mb - 120) * 30); // 0 to 240 in 8 minibeats
    }

    const uint16_t *town_tree[16] = {town_tree0, town_tree1, town_tree2, town_tree3, town_tree4, town_tree5, town_tree6, town_tree7, town_tree8, town_tree9, town_tree10, town_tree11, town_tree12, town_tree13, town_tree14, town_tree15};
    byte tree_num;
    boolean has_weather = false; // if true, overlaps the weather onto all panes. default false
    const uint8_t *weather;
    byte season; // 0 = summer, 1 = fall, 2 = winter, 3 = springssss

    // determine which tree to display
    if ((mb >= 0 && mb < 16) || (mb >= (4 * 16) && mb < (5 * 16))) { // summer
      season = 0;
      tree_num = 0;
      if (mb == 10 || mb == 11 || mb == 74 || mb == 75 ) {
        tree_num = 1;
      } else if (mb == 12 || mb == 13 || mb == 76 || mb == 77) {
        tree_num = 2;
      } else if (mb == 14 || mb == 15 || mb == 78 || mb == 79) {
        tree_num = 3;
      }
      if (mb >= (4 * 16) && mb < (5 * 16)) {
        has_weather = true;
      }
    } else if ((mb >= 16 && mb < (2 * 16)) || (mb >= (5 * 16) && mb < (6 * 16))) { // fall
      season = 1;
      tree_num = 4;
      if (mb == 26 || mb == 27 || mb == 90 || mb == 91 ) {
        tree_num = 5;
      } else if (mb == 28 || mb == 29 || mb == 92 || mb == 93) {
        tree_num = 6;
      } else if (mb == 30 || mb == 31 || mb == 94 || mb == 95) {
        tree_num = 7;
      }
      weather = false;
    } else if ((mb >= (2 * 16) && mb < (3 * 16)) || (mb >= (6 * 16) && mb < (7 * 16))) { // winter
      season = 2;
      tree_num = 8;
      if (mb == 42 || mb == 43 || mb == 106 || mb == 107) {
        tree_num = 9;
      } else if (mb == 44 || mb == 45 || mb == 108 || mb == 109) {
        tree_num = 10;
      } else if (mb == 46 || mb == 47 || mb == 110 || mb == 111) {
        tree_num = 11;
      }
      has_weather = true;
    } else if ((mb >= (3 * 16) && mb < (4 * 16)) || (mb >= (7 * 16) && mb < (8 * 16))) { // spring
      season = 3;
      tree_num = 12;
      if (mb == 58 || mb == 59 || mb == 123 || mb == 124 ) {
        tree_num = 13;
      } else if (mb == 60 || mb == 61 || mb == 125 || mb == 126) {
        tree_num = 14;
      } else if (mb == 62 || mb == 63 || mb == 127 || mb == 128) {
        tree_num = 15;
      }
      if (mb >= (3 * 16) && mb < (4 * 16)) {
        has_weather = true;
      }
    }

    // shifting the weather image to show motion
    if (has_weather) { // true
      if (season == 0 || season == 1 || season == 3) { // summer, fall, spring
        if (mb % 3 == 0) {
          weather = rain0;
        } else if (mb % 3 == 1) {
          weather = rain1;
        } else { // mb%3 == 2
          weather = rain2;
        }
      } else if (season == 2) { // winter
        if (mb % 8 < 4) {
          weather = snow0;
        } else { // mb%8 >= 4
          weather = snow1;
        }
      }
    }

    // create char panes
    // if there is weather, all char panes need to be overlapped
    if (has_weather) {
      // panes need to be overlapped with weather
      uint8_t pane[8];
      for (int k = 0; k < 6; k++) { // there are six panes for the town tree
        for (int i = 0; i < 8; i++) {
          byte col = (2 - (k % 3)) * 5; // columm or reverseIndex for IntToBinSubStrToDec()
          uint8_t town_tree_irow = IntToBinSubStrToDec((uint16_t)pgm_read_dword(town_tree[tree_num] + i + 8 * (k / 3)), col, 5); // note: this is the decimal representation
          uint8_t weather_irow = IntToBinSubStrToDec((uint8_t)pgm_read_dword(weather + i), 0, 5);
          pane[i] = town_tree_irow | weather_irow; // overlap the two images
        }
        lcd.createChar(k, pane);
      }
    } else {
      CreateCharPane16(town_tree[tree_num], 15, 0, 10, 31, 0);
      CreateCharPane16(town_tree[tree_num], 15, 0, 05, 31, 1);
      CreateCharPane16(town_tree[tree_num], 15, 0, 00, 31, 2);
      CreateCharPane16(town_tree[tree_num], 15, 8, 10, 31, 3);
      CreateCharPane16(town_tree[tree_num], 15, 8, 05, 31, 4);
      CreateCharPane16(town_tree[tree_num], 15, 8, 00, 31, 5);
    }

    // create outdoor decor and weather (if necessary)
    // if there is weather, all char panes need to be overlapped
    if (season == 0) { // summer
      if (has_weather) {
        uint8_t pane[8];
        for (int i = 0; i < 8; i++) {
          uint8_t grass_irow = IntToBinSubStrToDec((uint8_t)pgm_read_dword(grass0 + i), 0, 5); // note: this is the decimal representation
          uint8_t weather_irow = IntToBinSubStrToDec((uint8_t)pgm_read_dword(weather + i), 0, 5);
          pane[i] = grass_irow | weather_irow; // overlap the two images
        }
        lcd.createChar(6, pane);
        CreateCharPane8(weather, 7, 0, 0, 31, 7);
      } else {
        CreateCharPane8(grass0, 7, 0, 0, 31, 6);
        CreateCharPane8(flower1, 7, 0, 0, 31, 7);
      }
    } else if (season == 1) { // fall
      if (has_weather) {
        uint8_t pane[8];
        for (int i = 0; i < 8; i++) {
          uint8_t grass_irow = IntToBinSubStrToDec((uint8_t)pgm_read_dword(grass0 + i), 0, 5); // note: this is the decimal representation
          uint8_t weather_irow = IntToBinSubStrToDec((uint8_t)pgm_read_dword(weather + i), 0, 5);
          pane[i] = grass_irow | weather_irow; // overlap the two images
        }
        lcd.createChar(6, pane);
        CreateCharPane8(weather, 7, 0, 0, 31, 7);
      } else {
        CreateCharPane8(grass0, 7, 0, 0, 31, 6);
        CreateCharPane8(grass1, 7, 0, 0, 31, 7);
      }
    } else if (season == 2) { // winter
      if (has_weather) {
        uint8_t pane[8];
        for (int i = 0; i < 8; i++) {
          uint8_t snowman_irow = IntToBinSubStrToDec((uint8_t)pgm_read_dword(snowman + i), 0, 5); // note: this is the decimal representation
          uint8_t weather_irow = IntToBinSubStrToDec((uint8_t)pgm_read_dword(weather + i), 0, 5);
          pane[i] = snowman_irow | weather_irow; // overlap the two images
        }
        lcd.createChar(6, pane);
        CreateCharPane8(weather, 7, 0, 0, 31, 7);
      } else {
        CreateCharPane8(snowman, 7, 0, 0, 31, 6);
        CreateCharPane8(snowman, 7, 0, 0, 31, 7); // overwrite pane 7
      }
    } else if (season == 3) { // spring
      if (has_weather) {
        uint8_t pane[8];
        for (int i = 0; i < 8; i++) {
          uint8_t flower0_irow = IntToBinSubStrToDec((uint8_t)pgm_read_dword(flower0 + i), 0, 5); // note: this is the decimal representation
          uint8_t weather_irow = IntToBinSubStrToDec((uint8_t)pgm_read_dword(weather + i), 0, 5);
          pane[i] = flower0_irow | weather_irow; // overlap the two images
        }
        lcd.createChar(6, pane);
        CreateCharPane8(weather, 7, 0, 0, 31, 7);
      } else {
        CreateCharPane8(flower0, 7, 0, 0, 31, 6);
        CreateCharPane8(flower1, 7, 0, 0, 31, 7);
      }
    }

    // print
    if (mb % 16 == 0) { // only write the images onto the LCD once after each measure
      lcd.clear();

      if (has_weather) { // has weather
        lcd.setCursor(0, 0);
        for (int i = 0; i < 16; i++) { // fill the entire upper row with the weather
          if (i == 6 || i == 7 || i == 8) {
            lcd.write(byte(i - 6)); // 0,1,2
          } else { // place down the weather
            lcd.write(7);
          }
        }
        lcd.setCursor(0, 1);
        for (int i = 0; i < 16; i++) { // fill the entire lower row with outdoor decor
          if (i == 6 || i == 7 || i == 8) {
            lcd.write(i - 3); // 3,4,5
          } else { // 'randomly' place the outdoor decor
            byte rand_num = random(0, 3);
            if (rand_num == 1) {
              lcd.write(6);
            } else if (rand_num == 2) {
              lcd.write(7);
            } else { // rand_num == 0
              lcd.write(7);
            }
          }
        }
      } else { // no weather
        lcd.setCursor(6, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2);
        lcd.setCursor(0, 1);
        for (int i = 0; i < 16; i++) { // fill the entire lower row with outdoor decor
          if (i == 6 || i == 7 || i == 8) {
            lcd.write(i - 3); // 3,4,5
          } else { // 'randomly' place the outdoor decor
            byte rand_num = random(0, 3);
            if (rand_num == 1) {
              lcd.write(6);
            } else if (rand_num == 2) {
              lcd.write(7);
            } else { // rand_num == 0
              lcd.print(F(" "));
            }
          }
        }
      }
    }
  } else if (mb < (74 * 16) ) { // up to the end
    // fade in kk slider. fade out when done
    mb = mb - (66 * 16);

    if (mb < 8) { // fade in
      analogWrite(V0Pin, (8 - mb) * 30); // 240 to 0 in 8 minibeats
    } else if (mb > 119) { // fade out
      analogWrite(V0Pin, (mb - 119) * 30); // 0 to 240 in 8 minibeats
    }

    const uint16_t *kk_slider[2] = {kk_slider0, kk_slider1};
    byte ind;
    if (mb % 8 < 4) {
      ind = 0;
    } else { // mb % 8 >= 4
      ind = 1;
    }

    // right most column of image
    int col[4] = { -1, -2, -3, -2};
    byte frame = (mb / 6) % 4;

    CreateCharPane16(kk_slider[ind], 15, 0, col[frame] + 12, 31, 0);
    CreateCharPane16(kk_slider[ind], 15, 0, col[frame] + 06, 31, 1);
    CreateCharPane16(kk_slider[ind], 15, 0, col[frame] + 00, 31, 2);
    CreateCharPane16(kk_slider[ind], 15, 8, col[frame] + 12, 31, 3);
    CreateCharPane16(kk_slider[ind], 15, 8, col[frame] + 06, 31, 4);
    CreateCharPane16(kk_slider[ind], 15, 8, col[frame] + 00, 31, 5);

    lcd.clear();

    if (mb < 108) { // display notes according to the song
      CreateCharPane8(music_note, 7, 0, 0, 31, 7);

      // treble
      // minibeats lenghts were altered so there was at least one minibeat between consecutive beats
      // all note lenghts ( (0 <= mb && mb < 8) || (10 <= mb && mb < 12) || (12 <= mb && mb < 14) || (14 <= mb && mb < 24) || (24 <= mb && mb < 26) || (26 <= mb && mb < 28) || (28 <= mb && mb < 30) || (30 <= mb && mb < 40) || (42 <= mb && mb < 44) || (44 <= mb && mb < 46) || (46 <= mb && mb < 56) || (58 <= mb && mb < 60) || (60 <= mb && mb < 62) || (62 <= mb && mb < 64) || (64 <= mb && mb < 80) || (86 <= mb && mb < 88) || (88 <= mb && mb < 90) || (90 <= mb && mb < 92) || (92 <= mb && mb < 94) || (94 <= mb && mb < 106) )
      // compile every other note into two separate values
      boolean treble_note1 = ( (0 <= mb && mb < 8) || (12 <= mb && mb < 14) || (24 <= mb && mb < 26) || (28 <= mb && mb < 30) || (42 <= mb && mb < 44) || (46 <= mb && mb < 56) || (60 <= mb && mb < 62) || (64 <= mb && mb < 80) || (88 <= mb && mb < 90) || (92 <= mb && mb < 94) );
      boolean treble_note2 = ( (10 <= mb && mb < 12) || (14 <= mb && mb < 24) || (26 <= mb && mb < 28) || (30 <= mb && mb < 40) || (44 <= mb && mb < 46) || (58 <= mb && mb < 60) || (62 <= mb && mb < 64) || (86 <= mb && mb < 88) || (90 <= mb && mb < 92) || (94 <= mb && mb < 106));

      // bass
      // minibeats lenghts were altered so there was at least one minibeat between consecutive beats
      // all note lenghts ( (0 <= mb && mb < 4) || (4 <= mb && mb < 8) || (10 <= mb && mb < 12) || (14 <= mb && mb < 16) || (18 <= mb && mb < 24) || (24 <= mb && mb < 28) || (28 <= mb && mb < 32) || (32 <= mb && mb < 36) || (36 <= mb && mb < 40) || (42 <= mb && mb < 44) || (46 <= mb && mb < 48) || (50 <= mb && mb < 56) || (56 <= mb && mb < 60) || (60 <= mb && mb < 64) || (64 <= mb && mb < 68) || (68 <= mb && mb < 72) || (74 <= mb && mb < 76) || (78 <= mb && mb < 80) || (80 <= mb && mb < 96) || (96 <= mb && mb < 100) || (102 <= mb && mb < 104) )
      // compile every other note into two separate values
      boolean bass_note1 = ( (0 <= mb && mb < 4) || (10 <= mb && mb < 12) || (18 <= mb && mb < 24) || (28 <= mb && mb < 32) || (36 <= mb && mb < 40) || (46 <= mb && mb < 48) || (56 <= mb && mb < 60) || (64 <= mb && mb < 68) || (74 <= mb && mb < 76) || (80 <= mb && mb < 96) || (102 <= mb && mb < 104) );
      boolean bass_note2 = ( (4 <= mb && mb < 8) || (14 <= mb && mb < 16) || (24 <= mb && mb < 28) || (32 <= mb && mb < 36) || (42 <= mb && mb < 44) || (50 <= mb && mb < 56) || (60 <= mb && mb < 64) || (68 <= mb && mb < 72) || (78 <= mb && mb < 80) || (96 <= mb && mb < 100) );

      lcd.setCursor(0, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2);
      if (treble_note1 || treble_note2) { // display nothing if no note is played
        byte k;
        if (treble_note1) { // display notes in treble_note1 on even panes
          k = 0;
        } else { // odd panes for treble_note2
          k = 1;
        }

        for (int i = 3; i < 16; i++) {
          if (i % 2 == k) {
            lcd.write(7);
          } else {
            lcd.print(F(" "));
          }
        }
      }

      lcd.setCursor(0, 1); lcd.write(3); lcd.write(4); lcd.write(5);
      if (bass_note1 || bass_note2) { // display nothing if no note is played
        byte k = (bass_note1 ? 0 : 1); // display notes in bass_note1 on even panes, odd panes for bass_note2
        for (int i = 3; i < 16; i++) {
          if (i % 2 != k) {
            lcd.write(7);
          } else {
            lcd.print(F(" "));
          }
        }
      }
    } else { // ending chime
      byte k;
      if (mb == 109) {
        CreateCharPane8(music_note, 7, 0, 0, 00, 6); // blank
        CreateCharPane8(music_note, 7, 0, 0, 31, 7);
        k = 0;
      } else if (mb == 110) {
        CreateCharPane8(music_note, 7, -4, 0, 31, 6);
        CreateCharPane8(music_note, 7, 05, 0, 31, 7);
        k = 1;
      } else if (mb >= 111) {
        CreateCharPane8(music_note, 7, 0, 0, 31, 6);
        CreateCharPane8(music_note, 7, 0, 0, 00, 7); // blank
        k = 2;
      }

      lcd.setCursor(0, 0); lcd.write(byte(0)); lcd.write(1); lcd.write(2);
      for (int i = 3; i < 15; i++) {
        if (i % 3 == k) {
          lcd.write(6);
        } else {
          lcd.print(F(" "));
        }
      }

      lcd.setCursor(0, 1); lcd.write(3); lcd.write(4); lcd.write(5);
      for (int i = 3; i < 15; i++) {
        if (i % 3 == k) {
          lcd.write(7);
        } else {
          lcd.print(F(" "));
        }
      }
    }
  }

  // bye bye, Mr. Rossetti
}

// Subfunction: IntToBinSubStrToDec
// returns the decimal value of the substring of a binary represented integer
//
// input:  integer = uint32_t, the number whose binary substring is to be extracted and given back in decimal representation
//         reverseIndex = signed char, indexing where from right to left is the positive direction
//         strLength = byte, the length of the binary substring to be extracted.
//                       note: strLength must be positive
// output: returns the decimal value of the substring of a binary represented integer
//
// note: We return a byte as the LCD can only show 5 pixels on each cell row
//       In other words, each cell row is a length 5 binary where 11111 = 31
//         is the max value possible.
//      In addition, since we wish to return a byte, the string length cannot be greater than 8.
// Yes, we will display using decimal values not binary.
// This subfuction allows us:
//   1. To store full size custom characters as arrays of big integers (less space compared to long character strings)
//   2. To grab a section off of the custom character for displaying it to the LCD
uint8_t IntToBinSubStrToDec(uint32_t integer, signed char reverseIndex, byte strLength) {
  if (abs(strLength) > 8) { // check if string length is greater than 8
    Serial.println(F("Error: strLength cannot be greater than 8"));
    while (true);
  }

  uint32_t MaxInteger = -1; // gives the max value for uint32_t
  uint32_t dec = 0; // pre-allocate the return value, default 0

  if (strLength == 0) {
    dec = 0;
  }
  if (strLength > 0) {
    if (reverseIndex + strLength < 0) { // if reverseIndex + strLength is not quite yet at the left start of binary number
      dec = 0;
    } else if (reverseIndex > 32) { // if reverseIndex goes beyond the length of the binary representation
      dec = 0;
    } else if (reverseIndex < 0) {
      dec = (integer << abs(reverseIndex)) & (MaxInteger >> (32 - strLength));
    } else { // reverseIndex >= 0
      dec = (integer >> reverseIndex) & (MaxInteger >> (32 - strLength));
    }
  }

  return dec;
}

// Subfunction: CreateCharPane#
// creates a 5x8 pane from the sprite image (pointer) using the top right index.
// calls lcd.createChar() and assigns it to the number given
//
// Requires: LiquidCrystal Library
void CreateCharPane8(const uint8_t *sprite, byte max_row, signed char row, signed char col, byte select, byte num) {
  if (num >= 8) { // check num
    Serial.println(F("Error: num must be an integer within [0,7]"));
    while (true);
  }
  if (select > 31) { // check select
    Serial.println(F("Error: select must be an integer within [0,31]"));
    while (true);
  }

  // pre-initialize as zeros
  uint8_t pane[8] = {0, 0, 0, 0, 0, 0, 0, 0};

  // recall: indices start at 0
  for (int i = 0; i < 8; i++) {
    if (row > -1 && row <= max_row) { // if row is within [0,max_row]
      pane[i] = IntToBinSubStrToDec((uint8_t)pgm_read_dword(sprite + row), col, 5) & select;
    }
    row++;
  }

  lcd.createChar(num, pane);
}
void CreateCharPane16(const uint16_t *sprite, byte max_row, signed char row, signed char col, byte select, byte num) {
  if (num >= 8) { // check num
    Serial.println(F("Error: num must be an integer within [0,7]"));
    while (true);
  }
  if (select > 31) { // check select
    Serial.println(F("Error: select must be an integer within [0,31]"));
    while (true);
  }

  // pre-initialize as zeros
  uint8_t pane[8] = {0, 0, 0, 0, 0, 0, 0, 0};

  // recall: indices start at 0
  for (int i = 0; i < 8; i++) {
    if (row > -1 && row <= max_row) { // if row is within [0,max_row]
      pane[i] = IntToBinSubStrToDec((uint16_t)pgm_read_dword(sprite + row), col, 5) & select;
    }
    row++;
  }

  lcd.createChar(num, pane);
}
void CreateCharPane32(const uint32_t *sprite, byte max_row, signed char row, signed char col, byte select, byte num) {
  if (num >= 8) { // check num
    Serial.println(F("Error: num must be an integer within [0,7]"));
    while (true);
  }
  if (select > 31) { // check select
    Serial.println(F("Error: select must be an integer within [0,31]"));
    while (true);
  }

  // pre-initialize as zeros
  uint8_t pane[8] = {0, 0, 0, 0, 0, 0, 0, 0};

  // recall: indices start at 0
  for (int i = 0; i < 8; i++) {
    if (row > -1 && row <= max_row) { // if row is within [0,max_row]
      pane[i] = IntToBinSubStrToDec((uint32_t)pgm_read_dword(sprite + row), col, 5) & select;
    }
    row++;
  }

  lcd.createChar(num, pane);
}
