// Program Memory Library allows us store variables into program (flash) memory
// as opposed to static random access memory (SRAM or dynamic memory)
// refer to:http://www.nongnu.org/avr-libc/user-manual/group__avr__pgmspace.html
#include <avr/pgmspace.h>

// Tone Library allows us to play up to 3 tones on one piezo
// Note: In this code we only play 2 tones because a third tone will mess with timer 0;
//       and thus functions such as millis() and delay()
// Refer to its examples
#include <Tone.h> // includes pitches

///* Note Types *///
// note: Count by 16th notes, rare for pieces to have 32nd notes
//       i.e. quarter note = 4 sixteenth notes, so quarter note plays for 4 minibeats
// Define notes and notes with dots
#define W 16
#define H  8 
#define Q  4 
#define E  2
#define S  1
#define W_DOT 24
#define H_DOT 12
#define Q_DOT  6
#define E_DOT  3
// ties will be handled by adding notes

///* Animal Crossing - K.K. Bossa *///
// Refer to the pdf in the same folder
// Note: took only the notes on top
//       each line is four measures
//       did not enter in grace notes
//       did not enter in consecutive notes for ties
// Changed some notes because they sounded weird on the arduino
const uint8_t KK_Bossa_bpm = 130; // beats per minute

const uint16_t KK_Bossa_Treble_Freq[] PROGMEM = {
  // PAGE 1
  REST, REST, REST, REST,
  REST, REST, REST, REST,
  NOTE_F5S, REST, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_E5, NOTE_F5S, REST, NOTE_D5, NOTE_B4, NOTE_D5, REST, NOTE_D5, NOTE_C5,
  NOTE_C5, NOTE_C5, NOTE_E5, NOTE_G5, NOTE_F5S, NOTE_D5, NOTE_A4, NOTE_D5, NOTE_B4, REST, REST, REST,
  // PAGE 2
  NOTE_F5S, REST, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_E5, NOTE_F5S, REST, NOTE_G5, NOTE_F5S, NOTE_E5, REST, NOTE_E5, NOTE_G5, NOTE_A5,
  NOTE_B5, REST, REST, NOTE_A5, NOTE_F5S, NOTE_D5, NOTE_E5, NOTE_F5S, REST,
  NOTE_F5S, REST, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_E5, NOTE_F5S, REST, NOTE_D5, NOTE_B4, NOTE_D5, REST, NOTE_D5, NOTE_C5,
  NOTE_C5, NOTE_C5, NOTE_E5, NOTE_G5, NOTE_F5S, NOTE_D5, NOTE_A4, NOTE_D5, NOTE_B4, REST, REST, REST,
  NOTE_F5S, REST, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_E5, NOTE_F5S, REST, NOTE_G5, NOTE_F5S, NOTE_E5, REST, NOTE_E5, NOTE_G5, NOTE_A5,
  // PAGE 3
  NOTE_B5, REST, REST, NOTE_A5S, NOTE_F5S, NOTE_D5, NOTE_E5, NOTE_F5S, REST, NOTE_G5, NOTE_G5, NOTE_F5S,
  NOTE_E5, REST, NOTE_F5S, NOTE_F5S, NOTE_E5, NOTE_D5, REST, NOTE_E5, NOTE_E5, NOTE_G5,
  NOTE_C5, REST, REST, NOTE_C5, NOTE_C5, NOTE_B4, NOTE_C5, NOTE_C5S, NOTE_D5, REST, NOTE_G5, NOTE_G5, NOTE_F5S,
  NOTE_E5, REST, REST, NOTE_F5, NOTE_F5S, NOTE_E5, NOTE_F5S, NOTE_D5, REST, REST, NOTE_D5,
  // PAGE 4
  NOTE_C5S, NOTE_G5, REST, NOTE_G5, NOTE_F5S, NOTE_C5, REST, NOTE_G5, NOTE_F5S, NOTE_E5, REST, NOTE_A5,
  NOTE_E5, NOTE_D5, REST, NOTE_F5S, REST, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_E5, NOTE_F5S, // LAST NOTE HAS A TIE ONTO THE NEXT MEASURE
  REST, NOTE_D5, NOTE_B4, NOTE_D5, REST, NOTE_D5, NOTE_C5, NOTE_C5, NOTE_C5, NOTE_E5, NOTE_G5, NOTE_F5S, NOTE_D5, NOTE_A4, NOTE_D5, NOTE_B4, // LAST NOTE HAS A TIE ONTO THE NEXT MEASURE
  REST, REST, REST, NOTE_F5S, REST, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_B4, NOTE_D5, NOTE_E5, NOTE_F5S, // LAST NOTE HAS A TIE ONTO THE NEXT MEASURE
  // PAGE 5
  REST, NOTE_G5, NOTE_F5S, NOTE_E5, REST, NOTE_E5, NOTE_G5, NOTE_A5, NOTE_B5, REST, REST, NOTE_A5, NOTE_F5S, NOTE_D5, NOTE_E5, NOTE_F5S, // LAST NOTE HAS A TIE ONTO THE NEXT MEASURE
  REST, NOTE_D4, NOTE_A4, NOTE_D5
};

const uint8_t KK_Bossa_Treble_Notes[] PROGMEM = {
  // PAGE 1
  W, W, W, W,
  W, W, W, W,
  H, E, E, E, E + H, E, E, E, E + H, E, E, E, E + H, E, E + S, E_DOT,
  H, E, E, E, E + Q, E, E + Q, E, E + Q, Q, H, W,
  // PAGE 2
  H, E, E, E, E + H, E, E, E, E + H, E, E, E, E + H, E, E, E, E,
  W, Q, E, E, E, E, E, E + W, W,
  H, E, E, E, E + H, E, E, E, E + H, E, E, E, E + H, E, E + S, E_DOT,
  H, E, E, E, E + Q, E, E + Q, E, E + Q, Q, H, W,
  H, E, E, E, E + H, E, E, E, E + H, E, E, E, E + H, E, E, E, E,
  // PAGE 3
  W, Q, E, E, E, E, E, E + W, H, E_DOT, S + E, E,
  W, H, E_DOT, S + E, E, W, H, E_DOT, S + E, E,
  W, Q, E, E, E_DOT, S + E, E, Q_DOT, E + H, H, E_DOT, S + E, E,
  W, Q, E, E, E_DOT, S + E, E, W, H, Q, Q,
  // PAGE 4
  Q_DOT, E + H + H, Q, Q, Q_DOT, E, Q, Q, Q_DOT, E, Q, Q,
  Q_DOT, E + H + H, H, H, E, E, E, E + H, E, E, E, E + H, // THIS ROW HAS 18 BEATS
  E, E, E, E + H, E, E + S, E_DOT, H, E, E, E, E + Q, E, E + Q, E, E + Q, // THIS ROW HAS 15 BEATS
  Q, H, W, H, E, E, E, E + H, E, E, E, E + H, // THIS ROW HAS 17 BEATS
  // PAGE 5
  E, E, E, E + H, E, E, E, E, W, Q, E, E, E, E, E, E + H_DOT, // THIS ROW HAS 17 BEATS
  S, S, S, S + W, // THIS ROW HAS 5 BEATS
};

const uint16_t KK_Bossa_Bass_Freq[] PROGMEM = {
  // PAGE 1
  NOTE_D4, NOTE_D4, REST, NOTE_D4, REST, NOTE_D4, REST, NOTE_D4, NOTE_D4, NOTE_D4, NOTE_D4, NOTE_D4, REST, NOTE_D4, REST, NOTE_F4, REST, NOTE_F4, NOTE_F4, NOTE_F4, NOTE_D4S,
  NOTE_D4, NOTE_D4, REST, NOTE_D4, REST, NOTE_D4, REST, NOTE_D4, NOTE_D4, NOTE_D4, NOTE_D4, NOTE_D4, REST, NOTE_D4, REST, NOTE_F4, REST, NOTE_D4S,
  NOTE_A3, NOTE_A3, REST, NOTE_A3, REST, NOTE_A3, REST, NOTE_A3, NOTE_A3, NOTE_A3, NOTE_D4, NOTE_D4, REST, NOTE_D4, REST, NOTE_D4, REST, NOTE_D4, NOTE_D4, NOTE_D4,
  NOTE_G3, NOTE_G3, REST, NOTE_G3, REST, NOTE_G3, REST, NOTE_C4, NOTE_C4, NOTE_C4, NOTE_A3, NOTE_A3, REST, NOTE_A3, REST, NOTE_F3, REST, NOTE_F3, NOTE_F3, NOTE_F3, NOTE_D3S,
  // PAGE 2
  NOTE_A3, NOTE_A3, REST, NOTE_A3, REST, NOTE_A3, REST, NOTE_A3, NOTE_A3, NOTE_A3, NOTE_D4, NOTE_D4, REST, NOTE_D4, REST, NOTE_C4S, REST, NOTE_C4S, NOTE_D3, NOTE_D3,
  NOTE_G3, NOTE_G3, REST, NOTE_G3, REST, NOTE_C4, REST, NOTE_C4, NOTE_C4, NOTE_C4, NOTE_A3, NOTE_A3, REST, NOTE_A3, REST, NOTE_F3S, REST, NOTE_F3S, NOTE_F3S, NOTE_F3S,
  NOTE_D4, NOTE_D4,
  NOTE_C4, NOTE_C4, NOTE_D4, NOTE_F4, NOTE_D4S,
  NOTE_D4, NOTE_D4, NOTE_C4S, NOTE_C4,
  // PAGE 3
  NOTE_E4, NOTE_C4, NOTE_D4,
  NOTE_B3, NOTE_B3, REST, NOTE_B3, REST, NOTE_E3, REST, NOTE_E3, NOTE_E3, NOTE_E3, NOTE_A3, NOTE_A3, REST, NOTE_A3, REST, NOTE_G3S, REST, NOTE_C3, NOTE_C3, NOTE_C3,
  NOTE_G3, NOTE_G3, REST, NOTE_G3, REST, NOTE_F3S, REST, NOTE_C4, NOTE_C4, NOTE_C4, NOTE_A3, NOTE_A3, REST, NOTE_A3, REST, NOTE_A3, REST, NOTE_A3, NOTE_A3, NOTE_A3,
  NOTE_B3, NOTE_B3, REST, NOTE_B3, REST, NOTE_E3, REST, NOTE_E3, NOTE_E3, NOTE_E3, NOTE_A3S, NOTE_A3S, REST, NOTE_D3, REST, NOTE_D3, REST, NOTE_A3, NOTE_C3, NOTE_C3,
  // PAGE 4
  NOTE_G3, NOTE_G3, REST, NOTE_G3, REST, NOTE_G3, REST, NOTE_G3, NOTE_G3, NOTE_G3, NOTE_G3, NOTE_G3, REST, NOTE_G3, REST, NOTE_A3, REST, NOTE_A3, NOTE_G3S, NOTE_G3S,
  NOTE_G3, NOTE_G3, REST, NOTE_G3, REST, NOTE_F3S, REST, NOTE_C4, NOTE_C4, NOTE_D4S, NOTE_A3, NOTE_A3, REST, NOTE_A3, REST, NOTE_A3, REST, NOTE_A3, NOTE_A3, NOTE_A3,
  NOTE_D4, NOTE_D4, REST, NOTE_D4, REST, NOTE_D4, REST, NOTE_D4, NOTE_D4, NOTE_D4, NOTE_G3, NOTE_G3, REST, NOTE_G3, REST, NOTE_G3, REST, NOTE_C4, NOTE_C4, NOTE_C4,
  NOTE_A3, NOTE_A3, REST, NOTE_A3, REST, NOTE_F3, REST, NOTE_F3, NOTE_F3, NOTE_D3, NOTE_A3, NOTE_A3, REST, NOTE_A3, REST, NOTE_A3, REST, NOTE_A3, NOTE_A3, NOTE_A3,
  // PAGE 5
  NOTE_D4, NOTE_D4, REST, NOTE_D4, REST, NOTE_C4S, REST, NOTE_C4S, NOTE_D3, NOTE_D3, NOTE_G3, NOTE_G3, REST, NOTE_G3, REST, NOTE_C4, NOTE_C4,
  NOTE_A3, REST, NOTE_A3, REST, NOTE_G2
};

const uint8_t KK_Bossa_Bass_Notes[] PROGMEM = {
  // PAGE 1
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, E, E, Q, Q, Q,
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, Q, H_DOT,
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, E, E + Q, Q, Q,
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, E, E, Q, Q, Q,
  // PAGE 2
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, E, Q_DOT, Q, Q,
  Q, Q, E, E, E, E, E, Q_DOT, Q, Q, Q, Q, E, E, E, E, E, Q_DOT, Q, Q,
  W + W, W + W,
  W, W, W, H, H,
  W + W, W, H, H,
  // PAGE 3
  W, W, W + W,
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, E, E + Q, Q, Q,
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, E, E + Q, Q, Q,
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, E, E + Q, Q, Q,
  // PAGE 4
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, E, E + Q, Q, Q,
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, E, E + Q, Q, Q,
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, E, E + Q, Q, Q,
  Q, Q, E, E, E, E, E, E + Q, Q, Q, Q, Q, E, E, E, E, E, E + Q, Q, Q,
  // PAGE 5
  Q, Q, E, E, E, E, E, Q_DOT, Q, Q, Q, Q, E, E, E, E, W,
  Q, E, E, Q, Q + W // THIS ROW HAS 8 BEATS
};
